var MigrationBackgroundAndroid = (function () {
	function MigrationBackgroundAndroid() {
		this.isRunning = false;
		this.stateSubscribers = [];
	}
	MigrationBackgroundAndroid.prototype.getBlobVersion = function () {
		return AndroidApi.getBlobVersion();
	};
	MigrationBackgroundAndroid.prototype.getLinkedUsername = function () {
        return  AndroidApi.getLinkedAccountUsername();
    };
    MigrationBackgroundAndroid.prototype.getLinkedBlobVersion  = function (callback) {
            //this is run asynchronously, so store the callback function
            MigrationBackgroundAndroid.getLinkedBlobVersionCallback = callback;
            AndroidApi.onGetLinkedAccountBlobVersion("MigrationBackgroundAndroid.getLinkedBlobVersionCallback");
        };
	MigrationBackgroundAndroid.prototype.saveBlobVersion = function (version, callback) {
	    if(console){
        	console.log("saving blob version " + version);
        }
	    //this is run asynchronously, so store the callback function
    	MigrationBackgroundAndroid.saveBlobVersionCallback = callback;
    	AndroidApi.onSaveBlobVersion(version, "MigrationBackgroundAndroid.saveBlobVersionCallback");
	};
	MigrationBackgroundAndroid.prototype.saveLinkedBlobVersion = function (version, callback) {
    	    if(console){
            	console.log("saving linked account blob version " + version);
            }
    	    //this is run asynchronously, so store the callback function
        	MigrationBackgroundAndroid.saveLinkedBlobVersionCallback = callback;
        	AndroidApi.onSaveLinkedAccountBlobVersion(version, "MigrationBackgroundAndroid.saveLinkedBlobVersionCallback");
    	};
	MigrationBackgroundAndroid.prototype.subscribeToState = function (callback) {
		this.stateSubscribers.push(callback);
		if (this.isRunning) {
			callback(this.isRunning);
		}
	};
	MigrationBackgroundAndroid.prototype.publishState = function (state) {
		this.isRunning = state;
		for (var _i = 0, _a = this.stateSubscribers; _i < _a.length; _i++) {
			var stateSubscriber = _a[_i];
			stateSubscriber(state);
		}
	};

	MigrationBackgroundAndroid.prototype.sendSegmentEvent = function (event, args) {
	    if(console){
	        console.log("Sending event " + event + ": " + JSON.stringify(args));
	    }
         AndroidApi.sendSegmentEvent(event, args ? JSON.stringify(args) : null);
    };

	return MigrationBackgroundAndroid;
}());

var FormfillMigrationBackgroundAndroid = (function () {
	function FormfillMigrationBackgroundAndroid() {
	}
	FormfillMigrationBackgroundAndroid.prototype.isEnabled = function () {
		return AndroidApi.isFormFillMigrationEnabled();
	};
	FormfillMigrationBackgroundAndroid.prototype.createCustomNoteType = function (title, fields, callback) {
	    if(console){
            console.log("creating custom note " + title);
        }
		//this is run asynchronously, so store the callback function
		FormfillMigrationBackgroundAndroid.createCustomNoteTypeCallback = callback;
		FormFillApi.onCreateCustomNoteType(title, JSON.stringify(fields), "FormfillMigrationBackgroundAndroid.createCustomNoteTypeCallback");
	};
	FormfillMigrationBackgroundAndroid.prototype.convertFormFillData = function (formFillData) {

            if(formFillData.custom_fields){
    		    textFieldName = "text";
    		    idFieldName = "cfid";
    		    altTextFieldName = "alttext";
    		    valueFieldName = "value";

    		    for(var i = 0; i < formFillData.custom_fields.length; ++i){
    		        var custom_field = formFillData.custom_fields[i];
    		        fieldBaseName = "customfield" + i;

    		        formFillData[fieldBaseName + textFieldName] = custom_field.text;
    		        formFillData[fieldBaseName + idFieldName] = custom_field.cfid;
    		        formFillData[fieldBaseName + altTextFieldName] = custom_field.alttext;
    		        formFillData[fieldBaseName + valueFieldName] = custom_field.value;
    		    }
    		}

    		formFillData.decprofilename = formFillData.profilename;

    		var entity = new FormFillType();
            for (var field in formFillData) {
                if (formFillData.hasOwnProperty(field)) {
                    entity[field] = formFillData[field];
                }
            }

            return entity;
    	};
    FormfillMigrationBackgroundAndroid.prototype.parseFormFills = function(formFillsJson){
    	    if(!formFillsJson) return [];
    	    var formFillsObj = JSON.parse(formFillsJson);
    	    if(!formFillsObj) return [];

            if(console){
                console.log("Migrating " + formFillsObj.length + " formfills");
            }

            entities = [];
            for (var i = 0; i < formFillsObj.length; i++) {
                var formFillData = formFillsObj[i];
                var entity = this.convertFormFillData(formFillData);
                entities.push(entity);
            }

    		return entities;
    }
	FormfillMigrationBackgroundAndroid.prototype.getFormfills = function () {
	    var formFillsJson = FormFillApi.getVaultFormFills();
	    return this.parseFormFills(formFillsJson);
	};
	FormfillMigrationBackgroundAndroid.prototype.getLinkedFormfills = function () {
        var formFillsJson = FormFillApi.getLinkedAccountVaultFormFills();
        return this.parseFormFills(formFillsJson);
    };
	FormfillMigrationBackgroundAndroid.prototype.saveNote = function (data, callback) {
	    if(console){
            console.log("saving note " + data.name);
        }
		//this is run asynchronously, so store the callback function
        FormfillMigrationBackgroundAndroid.saveNoteCallback = callback;
        FormFillApi.onSaveNote(JSON.stringify(data), "FormfillMigrationBackgroundAndroid.saveNoteCallback");
	};
	return FormfillMigrationBackgroundAndroid;
}());